@@ -26,11 +26,21 @@
 #
 # SPDX-License-Identifier: curl
 
-readonly ROOTDIR=$(CDPATH= cd -- "$(dirname -- "$0")/.." && pwd)
+ROOTDIR=$(CDPATH=$(cd -- "$(dirname -- "$0")/.." && pwd))
+readonly ROOTDIR
 export PATH="${ROOTDIR}:${PATH}"
 
 readonly WCURL_CMD="wcurl --dry-run "
 
+oneTimeSetUp()
+{
+    if ! assertContains "Check compatibility" "test" "test"; then
+        echo "Error: shunit2 version 2.1.8 or higher is required."
+        echo "Please install a compatible version of shunit2."
+        exit 1
+    fi
+}
+
 debug()
 {
     if [ -n "${DEBUG}" ]; then
@@ -67,8 +77,9 @@ testParallelIfMoreThanOneUrl()
 {
     # TODO: This test is wrong for curl 7.65 or older, since --parallel was only introduced in 7.66.
     #       We should check curl's version and skip this test instead.
-    urls='example.com/1 example.com/2'
-    ret=$(${WCURL_CMD} ${urls})
+    url_1='example.com/1'
+    url_2='example.com/2'
+    ret=$(${WCURL_CMD} ${url_1} ${url_2})
     assertContains "Verify whether 'wcurl' uses '--parallel' if more than one url is provided" "${ret}" '--parallel'
 }
 
@@ -82,7 +93,7 @@ testEncodingWhitespace()
 testDoubleDash()
 {
     params='example.com --curl-options=abc'
-    ret=$(${WCURL_CMD} -- ${params})
+    ret=$(${WCURL_CMD} -- "${params}")
     assertTrue "Verify whether 'wcurl' accepts '--' without erroring" "$?"
     assertContains "Verify whether 'wcurl' considers everywhing after '--' a url" "${ret}" '--curl-options=abc'
 }
@@ -90,7 +101,7 @@ testDoubleDash()
 testCurlOptions()
 {
     params='example.com --curl-options=--foo --curl-options --bar'
-    ret=$(${WCURL_CMD} ${params})
+    ret=$(${WCURL_CMD} "${params}")
     assertTrue "Verify 'wcurl' accepts '--curl-options' with and without trailing '='" "$?"
     assertContains "Verify 'wcurl' correctly passes through --curl-options=<option>" "${ret}" '--foo'
     assertContains "Verify 'wcurl' correctly passes through --curl-options <option>" "${ret}" '--bar'
@@ -98,8 +109,10 @@ testCurlOptions()
 
 testNextAmount()
 {
-    urls='example.com/1 example.com/2 example.com3'
-    ret=$(${WCURL_CMD} ${urls})
+    url_1='example.com/1'
+    url_2='example.com/2'
+    url_3='example.com3'
+    ret=$(${WCURL_CMD} ${url_1} ${url_2} ${url_3})
     next_count=$(printf '%s' "${ret}" | grep -c -- --next)
     assertEquals "Verify whether 'wcurl' includes '--next' for every url besides the first" "${next_count}" "2"
 }
@@ -108,7 +121,7 @@ testUrlStartingWithDash()
 {
     url='-example.com'
     ret=$(${WCURL_CMD} ${url} 2>&1)
-    assertFalse "Verify wether 'wcurl' considers an URL starting with '-' as an option" "$?"
+    assertFalse "Verify whether 'wcurl' considers an URL starting with '-' as an option" "$?"
     assertEquals "${ret}" "Unknown option: '-example.com'."
 }
 
@@ -120,6 +133,14 @@ testOutputFileName()
     assertContains "Verify whether 'wcurl' correctly sets a custom output filename" "${ret}" 'test filename'
 }
 
+testOutputFileNameWithoutSpaces()
+{
+    url='example.com'
+    ret=$(${WCURL_CMD} -o"test filename" ${url} 2>&1)
+    assertContains "Verify whether 'wcurl' correctly sets --output" "${ret}" '--output'
+    assertContains "Verify whether 'wcurl' correctly sets --output with the correct filename" "${ret}" 'test filename'
+}
+
 testOutputFileNameRepeatedOption()
 {
     url='example.com'
@@ -142,43 +163,50 @@ testUrlDefaultNameTrailingSlash()
     assertContains "Verify whether 'wcurl' chooses the correct default filename when there's no path in the URL and the URl ends with a slash" "${ret}" 'index.html'
 }
 
-testUrlDecodingWhitespaces()
+testUrlDecodingWhitespace()
 {
     url='example.com/filename%20with%20spaces'
     ret=$(${WCURL_CMD} ${url} 2>&1)
-    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespaces in URLs" "${ret}" 'filename with spaces'
+    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespace in URLs" "${ret}" 'filename with spaces'
 }
 
-testUrlDecodingWhitespacesTwoFiles()
+testUrlDecodingWhitespaceTwoFiles()
 {
     url='example.com/filename%20with%20spaces'
     url_2='example.com/filename2%20with%20spaces'
     ret=$(${WCURL_CMD} ${url} ${url_2} 2>&1)
-    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespaces in URLs" "${ret}" 'filename with spaces'
-    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespaces in URLs" "${ret}" 'filename2 with spaces'
+    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespace in URLs" "${ret}" 'filename with spaces'
+    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespace in URLs" "${ret}" 'filename2 with spaces'
 }
 
 testUrlDecodingDisabled()
 {
     url='example.com/filename%20with%20spaces'
     ret=$(${WCURL_CMD} --no-decode-filename ${url} 2>&1)
-    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespaces in URLs" "${ret}" 'filename%20with%20spaces'
+    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespace in URLs" "${ret}" 'filename%20with%20spaces'
 }
 
-testUrlDecodingWhitespacesQueryString()
+testUrlDecodingWhitespaceQueryString()
 {
     url='example.com/filename%20with%20spaces?query=string'
-    ret=$(${WCURL_CMD} ${url} 2>&1)
-    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespaces in URLs with query strings" "${ret}" 'filename with spaces'
+    ret=$(${WCURL_CMD} "${url}" 2>&1)
+    assertContains "Verify whether 'wcurl' successfully decodes percent-encoded whitespace in URLs with query strings" "${ret}" 'filename with spaces'
 }
 
-testUrlDecodingWhitespacesTrailingSlash()
+testUrlDecodingWhitespaceTrailingSlash()
 {
     url='example.com/filename%20with%20spaces/'
     ret=$(${WCURL_CMD} ${url} 2>&1)
     assertContains "Verify whether 'wcurl' successfully uses the default filename when the URL ends with a slash" "${ret}" 'index.html'
 }
 
+testUrlDecodingBackslashes()
+{
+    url='example.com/filename%5Cwith%2Fbackslashes%5c%2f'
+    ret=$(${WCURL_CMD} ${url} 2>&1 | tr '\n' ' ')
+    assertContains "Verify whether 'wcurl' successfully uses the default filename when the URL ends with a slash" "${ret}" '--output filename%5Cwith%2Fbackslashes%5c%2f'
+}
+
 # Test decoding a bunch of different languages (that don't use the latin
 # alphabet), we could split each language on its own test, but for now it
 # doesn't make a difference.
@@ -217,4 +245,5 @@ testUrlDecodingNonLatinLanguages()
 ## - Options are the same for all URLs (except --next)
 ## - URLs beginning with '-' (with and without using '--')
 
+# shellcheck disable=SC1091
 . shunit2
